@@ -1,4 +1,4 @@
-/* $Id: weather-http.c 7588 2009-06-16 07:06:04Z colin $ */
+/* $Id: weather-http.c 7747 2009-07-19 20:55:22Z colin $ */
 /*
  * Copyright (c) 2007 Nick Schermer <nick@xfce.org>
  *
@@ -32,6 +32,8 @@
 #include <stdlib.h>
 #include <netinet/in.h>
 #include <resolv.h>
+#include <signal.h>
+#include <setjmp.h>
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -123,23 +125,19 @@ weather_http_receive_data_check (WeatherConnection *connection,
 static void refresh_resolvers(void)
 {
 #ifdef G_OS_UNIX
-	static time_t resolv_conf_changed = (time_t)NULL;
-	struct stat s;
-
-	/* This makes the glibc re-read resolv.conf, if it changed
-	 * since our startup. 
-	 * Why doesn't the glibc do it by itself?
-	 */
-	if (stat("/etc/resolv.conf", &s) == 0) {
-		if (s.st_mtime > resolv_conf_changed) {
-			resolv_conf_changed = s.st_mtime;
-			res_init();
-		}
-	} /* else
-		we'll have bigger problems. */
+	res_init();
 #endif /*G_OS_UNIX*/
 }
 
+#ifdef G_OS_UNIX
+static sigjmp_buf jmpenv;
+
+static void timeout_handler(gint sig)
+{
+	siglongjmp(jmpenv, 1);
+}
+#endif /*G_OS_UNIX*/
+
 static gboolean
 weather_http_receive_data_idle (gpointer user_data)
 {
@@ -147,11 +145,25 @@ weather_http_receive_data_idle (gpointer user_data)
   gchar               buffer[1024];
   gint                bytes, n, m;
   gchar              *request;
-  struct hostent     *host;
-  struct sockaddr_in  sockaddr;
+  
+  struct addrinfo     h, *r, *a;
+  const gchar         *port = NULL;
+  gint                err;
+
   const gchar        *p;
   GTimeVal            timeout;
+#ifdef G_OS_UNIX
+  void (*prev_handler)(gint);
 
+  alarm(0);
+  prev_handler = signal(SIGALRM, timeout_handler);
+  if (sigsetjmp(jmpenv, 1)) {
+	  alarm(0);
+	  signal(SIGALRM, prev_handler);
+	  connection->status = STATUS_TIMEOUT;
+	  return FALSE;
+  }
+#endif
   /* set the current time */
   g_get_current_time (&timeout);
 
@@ -159,11 +171,33 @@ weather_http_receive_data_idle (gpointer user_data)
   refresh_resolvers();
 
   /* try to get the hostname */
-  host = gethostbyname (connection->proxy_host ? connection->proxy_host : connection->hostname);
-  if (G_UNLIKELY (host == NULL))
+#ifdef G_OS_UNIX
+  alarm(WEATHER_MAX_CONN_TIMEOUT);
+#endif
+
+  memset(&h, 0, sizeof(h));
+  h.ai_family = AF_INET;
+  h.ai_socktype = SOCK_STREAM;
+  h.ai_protocol = IPPROTO_TCP;
+  
+  if (connection->proxy_port)
+  	port = g_strdup_printf("%d", connection->proxy_port);
+  else
+  	port = g_strdup("80");
+  
+  err = getaddrinfo(connection->proxy_host ? connection->proxy_host : connection->hostname,
+  		port, &h, &r);
+
+  g_free(port);
+#ifdef G_OS_UNIX
+  alarm(0);
+  signal(SIGALRM, prev_handler);
+#endif
+
+  if (G_UNLIKELY (err != 0))
     {
       /* display error */
-      g_message (_("Failed to get the hostname. Retry in %d seconds."),
+      g_message (_("Failed to get the hostname %s. Retry in %d seconds."), gai_strerror(err),
                  WEATHER_RESCHEDULE_TIMEOUT / 1000);
 
       /* try again later */
@@ -176,7 +210,31 @@ weather_http_receive_data_idle (gpointer user_data)
     return FALSE;
 
   /* open the socket */
-  connection->fd = socket (PF_INET, SOCK_STREAM, 0);
+  
+  for (a = r; a != NULL; a = a->ai_next) {
+    connection->fd = socket (a->ai_family, a->ai_socktype, a->ai_protocol);
+    if (connection->fd < 0) {
+      err = errno;
+      continue;
+    }
+#ifdef G_OS_UNIX
+    signal(SIGALRM, timeout_handler);
+    alarm(WEATHER_MAX_CONN_TIMEOUT);
+#endif
+    m = connect (connection->fd, a->ai_addr, a->ai_addrlen);
+#ifdef G_OS_UNIX
+    alarm(0);
+    signal(SIGALRM, prev_handler);
+#endif
+    if (m == 0)
+      break;
+    else 
+      err = errno;
+
+    if (weather_http_receive_data_check (connection, timeout))
+      break;
+  }
+
   if (G_UNLIKELY (connection->fd < 0))
     {
       /* display warning */
@@ -188,17 +246,6 @@ weather_http_receive_data_idle (gpointer user_data)
       return FALSE;
     }
 
-  if (weather_http_receive_data_check (connection, timeout))
-    return FALSE;
-
-  /* complete the host information */
-  sockaddr.sin_family = PF_INET;
-  sockaddr.sin_port = htons (connection->proxy_port ? connection->proxy_port : 80);
-  sockaddr.sin_addr = *((struct in_addr *)host->h_addr);
-  memset(&(sockaddr.sin_zero), '\0', 8);
-
-  /* open a connection with the host */
-  m = connect (connection->fd, (struct sockaddr *)&sockaddr, sizeof(struct sockaddr));
   if (G_UNLIKELY (m < 0))
     {
       /* display warning */
@@ -342,7 +389,7 @@ weather_http_receive_data_destroyed (gpointer user_data)
   WeatherConnection *connection = user_data;
 
   /* close the socket */
-  if (connection->fd != -1)
+  if (connection->fd >= 0)
     {
       close (connection->fd);
       connection->fd = -1;
@@ -425,7 +472,7 @@ weather_http_receive_data (const gchar  *hostname,
   connection->cb_user_data = user_data;
   connection->status = STATUS_NOT_EXECUTED;
   connection->received = NULL;
-  connection->fd = 0;
+  connection->fd = -1;
   connection->counter = 1;
 
   /* start idle function */
