@@ -1,4 +1,4 @@
-/*  $Id: weather.c 7588 2009-06-16 07:06:04Z colin $
+/*  $Id: weather.c 7748 2009-07-19 20:57:42Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  *
@@ -62,11 +62,13 @@ check_envproxy (gchar **proxy_host,
 
   tmp = strstr (env_proxy, "://");
 
-  if (!tmp || strlen (tmp) < 3)
+  if (!tmp)
+    tmp = env_proxy;
+  else if (strlen (tmp) >= 3)
+    env_proxy = tmp + 3;
+  else
     return FALSE;
 
-  env_proxy = tmp + 3;
-
   /* we don't support username:password so return */
   tmp = strchr (env_proxy, '@');
   if (tmp)
@@ -170,6 +172,9 @@ make_label (xml_weather    *weatherdata,
     case WIND_GUST:
       value = translate_wind_speed (rawvalue, unit);
       break;
+    case BAR_D:
+      value = g_strdup(translate_bard(rawvalue));
+      break;
     default:
       value = NULL;
       break;
@@ -325,7 +330,12 @@ cb_update (gboolean  succeed,
 
   if (G_LIKELY (succeed && result))
     {
-      doc = xmlParseMemory (result, strlen (result));
+      if (g_utf8_validate(result, -1, NULL)) {
+	/* force parsing as UTF-8, the XML encoding header may lie */
+	doc = xmlReadMemory (result, strlen (result), NULL, "UTF-8", 0);
+      } else {
+	doc = xmlParseMemory (result, strlen(result));
+      }
 
       g_free (result);
 
@@ -447,6 +457,16 @@ xfceweather_read_config (XfcePanelPlugin  *plugin,
       data->location_code = g_strdup (value);
     }
 
+  value = xfce_rc_read_entry (rc, "loc_name", NULL);
+
+  if (value)
+    {
+      if (data->location_name)
+        g_free (data->location_name);
+
+      data->location_name = g_strdup (value);
+    }
+
   if (xfce_rc_read_bool_entry (rc, "celcius", TRUE))
     data->unit = METRIC;
   else
@@ -535,6 +555,9 @@ xfceweather_write_config (XfcePanelPlugin  *plugin,
   if (data->location_code)
     xfce_rc_write_entry (rc, "loc_code", data->location_code);
 
+  if (data->location_name)
+    xfce_rc_write_entry (rc, "loc_name", data->location_name);
+
   xfce_rc_write_bool_entry (rc, "proxy_fromenv", data->proxy_fromenv);
 
   if (data->proxy_host)
@@ -632,6 +655,22 @@ cb_click (GtkWidget      *widget,
   return FALSE;
 }
 
+static gboolean
+cb_scroll (GtkWidget      *widget,
+           GdkEventScroll *event,
+           gpointer        user_data)
+{
+  xfceweather_data *data = (xfceweather_data *) user_data;
+
+  if (event->direction == GDK_SCROLL_UP || event->direction == GDK_SCROLL_DOWN)
+    {
+      gtk_scrollbox_next_label(GTK_SCROLLBOX(data->scrollbox));
+    }
+
+  return FALSE;
+}
+
+
 static void
 mi_click (GtkWidget *widget,
           gpointer   user_data)
@@ -793,6 +832,8 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
 
   g_signal_connect (G_OBJECT (data->tooltipbox), "button-press-event",
                     G_CALLBACK (cb_click), data);
+  g_signal_connect (G_OBJECT (data->tooltipbox), "scroll-event",
+                    G_CALLBACK (cb_scroll), data);
   gtk_widget_add_events(data->scrollbox, GDK_BUTTON_PRESS_MASK);
 
   /* add refresh button to right click menu, for people who missed the middle mouse click feature */
@@ -865,6 +906,7 @@ xfceweather_free (XfcePanelPlugin  *plugin,
 
   /* Free chars */
   g_free (data->location_code);
+  g_free (data->location_name);
   g_free (data->proxy_host);
 
   /* Free Array */
